#!/bin/sh
. /root/gstreamer-examples/func.sh

init_dev
[ $? -ne 0 ] && exit 1

guess_param


echo "select resolution"
echo "================="
echo "1 = 2592 x 1944 Full"
echo "2 = 2048 x 1536 QXGA, with ROI"
echo "3 = 1600 x 1200 UXGA, with ROI"
echo "4 = 1280 x 1024 SXGA, with ROI"
echo "5 = 1024 x 768  XGA, with Skip2 and ROI"
echo "6 = 800  x 600  SVGA, with Skip2 and ROI"
echo "7 = 640  x 480  VGA, with Skip4 and ROI"
echo "8 = 1920 x 1080 FullHD, with ROI"
echo "9 = 1280 x 720  HD, with Skip2 and ROI"
read RESOLUTION
echo "Your select = $RESOLUTION"
case $RESOLUTION in
  "1") SENSOR_RES="2592x1944"; GRAB_RES="2592x1944"; FRAME_SIZE="width=2592,height=1944"; OFFSET_SENSOR="(4,4)"; CTLS=(-c column_binning=0);;
  "2") SENSOR_RES="2048x1536"; GRAB_RES="2048x1536"; FRAME_SIZE="width=2048,height=1536"; OFFSET_SENSOR="(272,204)"; CTLS=(-c column_binning=0);;
  "3") SENSOR_RES="1600x1200"; GRAB_RES="1600x1200"; FRAME_SIZE="width=1600,height=1200"; OFFSET_SENSOR="(496,372)"; CTLS=(-c column_binning=0);;
  "4") SENSOR_RES="1280x1024"; GRAB_RES="1280x1024"; FRAME_SIZE="width=1280,height=1024"; OFFSET_SENSOR="(656,460)"; CTLS=(-c column_binning=0);;
  "5") SENSOR_RES="2048x1536"; GRAB_RES="1024x768"; FRAME_SIZE="width=1024,height=768"; OFFSET_SENSOR="(272,204)"; CTLS=(-c column_binning=1);;
  "6") SENSOR_RES="1600x1200"; GRAB_RES="800x600"; FRAME_SIZE="width=800,height=600"; OFFSET_SENSOR="(496,372)"; CTLS=(-c column_binning=1);;
  "7") SENSOR_RES="2560x1920"; GRAB_RES="640x480"; FRAME_SIZE="width=640,height=480"; OFFSET_SENSOR="(16,12)"; CTLS=(-c column_binning=1);;
  "8") SENSOR_RES="1920x1080"; GRAB_RES="1920x1080"; FRAME_SIZE="width=1920,height=1080"; OFFSET_SENSOR="(336,432)"; CTLS=(-c column_binning=0);;
  "9") SENSOR_RES="2560x1440"; GRAB_RES="1280x720"; FRAME_SIZE="width=1280,height=720"; OFFSET_SENSOR="(16,252)"; CTLS=(-c column_binning=1);;
  *) SENSOR_RES="2592x1944"; GRAB_RES="2592x1944"; FRAME_SIZE="width=2592,height=1944"; OFFSET_SENSOR="(4,4)"; CTLS=(-c column_binning=0);;
esac


echo " ================================================================================="
echo " starting gstreamer with $CAM_BW_FMT Source ..."
echo " read $FRAME_SIZE (offset x,y=$OFFSET_SENSOR) and write to framebuffer"
echo " =============================================================="
echo ""
echo " configure CSI1 (CAM1 port) with media_control"
echo " ================================================"

setup-pipeline-csi1 -f $CAM_BW_FMT -s $GRAB_RES -o $OFFSET_SENSOR -c $SENSOR_RES


echo ""
echo " configure camera @CAM1 with v4l2_control"
echo " ========================================"

v4l2-ctl "${CTLS[@]}" -d "$CAM_DEVICE"
$V4L2_CTRL_CAM1_BW
# Reduce the "exposure" parameter for a higher frame rate


echo ""
echo " start gstreamer, break with ctl-C"
echo " ================================="

gst-launch-1.0 \
	v4l2src device=$VID_DEVICE ! \
	video/x-raw,format=GRAY8,depth=8,$FRAME_SIZE ! \
	videoconvert n-threads=4 ! \
	waylandsink sync=false

CTLS=( -c column_binning=0 )
v4l2-ctl "${CTLS[@]}" -d "$CAM_DEVICE"