#!/bin/sh

VID_DEVICE_1="/dev/video-csi1"
CAM_DEVICE_1="/dev/cam-csi1"
CAM_ENT_1="$(cat /sys/class/video4linux/$(readlink /dev/cam-csi1)/name)"
CAM_NAME_1="$(echo ${CAM_ENT_1} | cut -d" " -f1)"
CAM_I2C_BUS_1="$(echo ${CAM_ENT_1} | cut -d" " -f2 | cut -b1)"
CAM_I2C_ADRESS_1="$(echo ${CAM_ENT_1} | cut -d" " -f2 | cut -b4-6)"

VID_DEVICE_2="/dev/video-csi2"
CAM_DEVICE_2="/dev/cam-csi2"
CAM_ENT_2="$(cat /sys/class/video4linux/$(readlink /dev/cam-csi2)/name)"
CAM_NAME_2="$(echo ${CAM_ENT_2} | cut -d" " -f1)"
CAM_I2C_BUS_2="$(echo ${CAM_ENT_2} | cut -d" " -f2 | cut -b1)"
CAM_I2C_ADRESS_2="$(echo ${CAM_ENT_2} | cut -d" " -f2 | cut -b4-6)"

echo ""
echo " camera 1 = $CAM_NAME_1 at i2c bus=$CAM_I2C_BUS_1 with i2c adress=0x$CAM_I2C_ADRESS_1 installed"
echo " capture device 1 @CAM1 = $(readlink -f /dev/video-csi1)"
echo " camera device 1 @CAM1 = $(readlink -f /dev/cam-csi1)"
echo " camera entity name 1 = $CAM_ENT_1"
echo " camera driver 1 = $CAM_NAME_1"
echo ""
echo " camera 2 = $CAM_NAME_2 at i2c bus=$CAM_I2C_BUS_2 with i2c adress=0x$CAM_I2C_ADRESS_2 installed"
echo " capture device 2 @CAM2 = $(readlink -f /dev/video-csi2)"
echo " camera device 2 @CAM2 = $(readlink -f /dev/cam-csi2)"
echo " camera entity name 2 = $CAM_ENT_2"
echo " camera driver 2 = $CAM_NAME_2"
echo ""

SENSOR_RES="2560x1920"
GRAB_RES="640x480"
FRAME_SIZE="width=640,height=480"

# set for FullHD
#SENSOR_RES="1920x1080"
#GRAB_RES="1920x1080"
#FRAME_SIZE="width=1920,height=1080"

OFFSET_SENSOR="(0,4)"
CAM_BW_FMT="Y8_1X8"
BW_FORMAT="raw,format=GRAY8,depth=8"


echo ""
echo " starting gstreamer with two GRAY Sources ..."
echo " read CAM1 from MIPI-CSI-2_CSI1 and CAM2 from MIPI-CSI-2_CSI2 with $GRAB_RES"
echo " write side by side to sink device"
echo " =================================================================================="
echo ""
echo " configure camera with v4l2_control"
echo " =================================="

setup-pipeline-csi1 -f $CAM_BW_FMT -s $GRAB_RES -o $OFFSET_SENSOR -c $SENSOR_RES
setup-pipeline-csi2 -f $CAM_BW_FMT -s $GRAB_RES -o $OFFSET_SENSOR -c $SENSOR_RES


echo ""
echo " configure camera @CAM1 with v4l2_control"
echo " ========================================"

v4l2-ctl -d $CAM_DEVICE_1 -c exposure=2240,analogue_gain=6000,dynamic_defect_pixel_correction=1
v4l2-ctl -d $CAM_DEVICE_2 -c exposure=2240,analogue_gain=6000,dynamic_defect_pixel_correction=1


echo ""
echo " start gstreamer"
echo " ==============="

gst-launch-1.0 videomixer name=mix sink_0::xpos=0 sink_0::ypos=0 sink_1::xpos=640 sink_1::ypos=0 ! waylandsink sync=false \
	v4l2src device=$VID_DEVICE_1 ! \
	video/x-$BW_FORMAT,$FRAME_SIZE,framerate=30/1 ! videoconvert n-threads=4 ! queue ! mix. \
	v4l2src device=$VID_DEVICE_2 ! \
	video/x-$BW_FORMAT,$FRAME_SIZE,framerate=30/1 ! videoconvert n-threads=4 ! queue ! mix.

# set for 30fps in FullHD
#gst-launch-1.0 \
#	v4l2src device=$VID_DEVICE_1 ! \
#	video/x-$BW_FORMAT,$FRAME_SIZE,framerate=30/1 ! videoconvert n-threads=4 ! queue ! \
#	waylandsink window-width=1280 window-height=720 &

#gst-launch-1.0 \
#	v4l2src device=$VID_DEVICE_2 ! \
#	video/x-$BW_FORMAT,$FRAME_SIZE,framerate=30/1 ! videoconvert n-threads=4 ! queue ! \
#	waylandsink window-width=1280 window-height=720
