#!/bin/sh
. /root/gstreamer-examples/func.sh

init_dev
[ $? -ne 0 ] && exit 1

echo " select resolution"
echo " ================="
echo " 1 = 2592 x 1944 Full"
echo " 2 = 1920 x 1080 FullHD, with Scale the Full Frame in the ISP"
echo " 3 = 1920 x 1080 FullHD, with Crop the Full Frame in the ISP"
echo " 4 = 1920 x 1080 FullHD, with Crop in the Sensor"
echo " 5 = 1280 x 720  HD, with Crop the Full Frame in the ISP"
echo " 6 = 1280 x 720  HD, with Skip2 and Crop in the Sensor"
echo " 7 = 1280 x 720  HD, with Skip2, Binning and Crop in the Sensor"
read RESOLUTION
echo " Your select = $RESOLUTION"
case $RESOLUTION in
  "1") SENSOR_RES="2592x1944"; FRAME_SIZE_FROM_ISP="width=2592,height=1944"; CTLS=(-c column_binning=0); MODE_VAR=$VM_017_AO062_2592x1944;;
  "2") SENSOR_RES="2592x1944"; FRAME_SIZE_FROM_ISP="width=1920,height=1080"; CTLS=(-c column_binning=0); MODE_VAR=$VM_017_AO062_2592x1944;;
  "3") SENSOR_RES="2592x1944"; FRAME_SIZE_FROM_ISP="width=1920,height=1080"; CTLS=(-c column_binning=0); MODE_VAR=$VM_017_AO062_2592x1944; V4L2_CTRL_CROP='v4l2-ctl --set-crop=top=336,left=432,width=1920,height=1080';;
  "4") SENSOR_RES="1920x1080"; FRAME_SIZE_FROM_ISP="width=1920,height=1080"; CTLS=(-c column_binning=0); MODE_VAR=$VM_017_AO062_1920x1080;;
  "5") SENSOR_RES="2592x1944"; FRAME_SIZE_FROM_ISP="width=1280,height=720"; CTLS=(-c column_binning=0); MODE_VAR=$VM_017_AO062_2592x1944; V4L2_CTRL_CROP='v4l2-ctl --set-crop=top=656,left=612,width=1280,height=720';;
  "6") SENSOR_RES="1280x720"; FRAME_SIZE_FROM_ISP="width=1280,height=720"; CTLS=(-c column_binning=0); MODE_VAR=$VM_017_AO062_1280x720;;
  "7") SENSOR_RES="1280x720"; FRAME_SIZE_FROM_ISP="width=1280,height=720"; CTLS=(-c column_binning=1); MODE_VAR=$VM_017_AO062_1280x720;;
  *) SENSOR_RES="2592x1944"; FRAME_SIZE_FROM_ISP="width=2592,height=1944"; CTLS=(-c column_binning=0); MODE_VAR=$VM_017_AO062_2592x1944;;
esac


echo " ================================================================================="
echo " starting gstreamer with COLOR (YUV) Source after ISP ..."
echo " read from sensor $SENSOR_RES, read after ISP $FRAME_SIZE_FROM_ISP, convert and write to framebuffer"
echo " ================================================================================="
echo ""
echo " choose right XML-File and JSON-File in Sensor0_Entry.cfg"
echo " ========================================================"

sed -i 's/^mode= .*/mode= '$MODE_VAR'/' $(readlink -f /opt/imx8-isp/bin/Sensor0_Entry.cfg)
XML_FILE=`grep -A1 mode.$MODE_VAR /opt/imx8-isp/bin/Sensor0_Entry.cfg | grep xml | cut -c12- | sed 's/"//g'`
JSON_FILE=`grep -A2 mode.$MODE_VAR /opt/imx8-isp/bin/Sensor0_Entry.cfg | grep json | cut -c12- | sed 's/"//g'`
echo " XML_File for ISP  = $XML_FILE"
echo " JSON_File for ISP = $JSON_FILE"


echo ""
echo " configure CSI1 (CAM1 port) with media_control"
echo " ============================================="

if [ -e /dev/phycam-deserializer-csi1 ] && [ -e /dev/phycam-serializer-csi1 ] ; then
	DESERIALIZER="$(cat /sys/class/video4linux/$(readlink /dev/phycam-deserializer-csi1)/name)"
	SERIALIZER="$(cat /sys/class/video4linux/$(readlink /dev/phycam-serializer-csi1)/name)"
	media-ctl -l "'${SERIALIZER}':1->'${DESERIALIZER}':0[1]"
fi


echo ""
echo " configure camera @CAM1 with v4l2_control"
echo " ========================================"

if [ "$(v4l2-ctl -d /dev/cam-csi1 --get-subdev-fmt 0 | grep "Mediabus Code" | sed 's/.*BUS_FMT_\([A-Z]*\).*/\1/g')" = "Y" ]; then
	$V4L2_CTRL_CAM1_BW_ISP
else
	$V4L2_CTRL_CAM1_COL_ISP
fi
$V4L2_CTRL_CROP

echo ""
echo " start gstreamer, break with ctl-C"
echo " ================================="

gst-launch-1.0 \
	v4l2src device=$VID_DEVICE ! \
	video/x-raw,format=YUY2,$FRAME_SIZE_FROM_ISP ! \
	waylandsink sync=true

CTLS=( -c column_binning=0 )
v4l2-ctl "${CTLS[@]}" -d "$CAM_DEVICE"