#!/bin/sh
. /root/gstreamer-examples/func.sh

init_dev
[ $? -ne 0 ] && exit 1

guess_param

HOST_IP="192.168.2.23"
echo ""
echo " Attention: Before start the script, set the right IP address for you Host!"
echo " In the moment is set HOST IP = $HOST_IP"
echo " =========================================================================="

echo " Choose the Encoder"
echo " =================="
echo " 1 = H.264 (IMX VPU-based AVC/H264 video encoder)"
echo " 2 = H.265 (IMX VPU-based HEVC video encoder)"
read ENCODER_TYP_NUMBER
echo " Your select = $ENCODER_TYP_NUMBER"
case $ENCODER_TYP_NUMBER in
  "1") ENCODER_TYP="H.264"; VPU_ENCODER="vpuenc_h264 ! rtph264pay"; SDP_FILE="VLC_Show_Networkstream_h264.sdp";;
  "2") ENCODER_TYP="H.265(HEVC)"; VPU_ENCODER="vpuenc_hevc ! rtph265pay"; SDP_FILE="VLC_Show_Networkstream_h265.sdp";;
  *) ENCODER_TYP="H.264"; VPU_ENCODER="vpuenc_h264 ! rtph264pay"; SDP_FILE="VLC_Show_Networkstream_h264.sdp";;
esac

echo ""
echo " Show the stream on HOST e.g with VLC-Player. Start the $SDP_FILE file"
echo " Note: Set the Device IP address in the VLC-SDP File!"
echo ""


echo " starting gstreamer with COLOR (BAYER, YUV or RGB depend of camera typ) Source ..."
echo " read $SENSOR_ARRAY_HD (offset x,y=$OFFSET_SENSOR_HD), convert/compress $ENCODER_TYP and write to network"
echo " ===================================================================================="
echo ""
echo " choose right XML-File and JSON-File in Sensor0_Entry.cfg"
echo " ========================================================"

MODE_VAR="0"					# 1280x720 resolution at VM-x16 and VM-x17
sed -i 's/^mode= .*/mode= '$MODE_VAR'/' $(readlink -f /opt/imx8-isp/bin/Sensor0_Entry.cfg)
XML_FILE=`grep -A1 mode.$MODE_VAR /opt/imx8-isp/bin/Sensor0_Entry.cfg | grep xml | cut -c12- | sed 's/"//g'`
JSON_FILE=`grep -A2 mode.$MODE_VAR /opt/imx8-isp/bin/Sensor0_Entry.cfg | grep json | cut -c12- | sed 's/"//g'`
echo " XML_File for ISP  = $XML_FILE"
echo " JSON_File for ISP = $JSON_FILE"


echo ""
echo " configure CSI1 (CAM1 port) with media_control"
echo " ============================================="

if [ -e /dev/phycam-deserializer-csi1 ] && [ -e /dev/phycam-serializer-csi1 ] ; then
	DESERIALIZER="$(cat /sys/class/video4linux/$(readlink /dev/phycam-deserializer-csi1)/name)"
	SERIALIZER="$(cat /sys/class/video4linux/$(readlink /dev/phycam-serializer-csi1)/name)"
	media-ctl -l "'${SERIALIZER}':1->'${DESERIALIZER}':0[1]"
fi


echo ""
echo " configure camera @CAM1 with v4l2_control"
echo " ========================================"

if [ "$(v4l2-ctl -d /dev/cam-csi1 --get-subdev-fmt 0 | grep "Mediabus Code" | sed 's/.*BUS_FMT_\([A-Z]*\).*/\1/g')" = "Y" ]; then
	$V4L2_CTRL_CAM1_BW_ISP
else
	$V4L2_CTRL_CAM1_COL_ISP
fi


echo ""
echo " start gstreamer, break with ctl-C"
echo " ================================="

gst-launch-1.0 \
	v4l2src device=$VID_DEVICE ! \
	video/x-raw,format=YUY2,framerate=60/1,pixel-aspect-ratio=1/1 ! \
	$VPU_ENCODER ! \
	udpsink host=$HOST_IP port=5200 sync=false

# Generate Testsource
#gst-launch-1.0 \
#	videotestsrc ! \
#	video/x-raw,format=YUY2,framerate=60/1,pixel-aspect-ratio=1/1 ! \
#	$VPU_ENCODER ! \
#	udpsink host=$HOST_IP port=5200 sync=false