#!/usr/bin/env python

'''
Sample show VideoCapture via v4l2 interface for PHYTEC phyCAM-M series.
Keys:
    q    - exit
'''

# Python 2/3 compatibility
import numpy as np
import cv2
import subprocess
import sys
import os
import time

prev_frame_time = 0
new_frame_time = 0
fps_average_count = 0
fps = 0

font                = cv2.FONT_HERSHEY_SIMPLEX
TopLeftCornerOfText = (10,30)
fontScale           = 1
fontColor           = (255,255,255)
lineType            = 2

print ("OpenCV version :  {0}".format(cv2.__version__))
face_cascade = cv2.CascadeClassifier('haarcascade_frontalface_default.xml')
eye_cascade = cv2.CascadeClassifier('haarcascade_eye.xml')

# read camera parameter at CSI1/CAM1 via ISI
CSI1_DEVICE = ("/dev/%s" % os.readlink( "/dev/video-csi1" ))
print (" capture device for CSI1 port =", CSI1_DEVICE)
CSI1_DEVICE_NUMBER = int(CSI1_DEVICE[10:].strip())	# .strip() -> cut the first 10 characters
print (" capture device number for CSI1 port = ", CSI1_DEVICE_NUMBER)

CAM1_DEVICE = ("/dev/%s" % os.readlink( "/dev/cam-csi1" ))
print (" camera device for CSI1 port  =", CAM1_DEVICE)

CAM_TYP = ("cat /sys/class/video4linux/%s/name" % os.readlink( "/dev/cam-csi1" ))
CAM_TYP = os.popen(str(CAM_TYP)).read()[:-1]				# cut the "\n" at the end
print (" Camera Typ =", CAM_TYP)
CAM_DRIVER = ( "echo %s | cut -d\" \" -f1" % CAM_TYP)
CAM_DRIVER = os.popen(str(CAM_DRIVER)).read()[:-1]			# cut the "\n" at the end
print (" Camera Driver Name =", CAM_DRIVER)

COLOR_FMT = "v4l2-ctl -d /dev/cam-csi1 --get-subdev-fmt 0 | grep \"Mediabus Code\" | sed \'s/.*BUS_FMT_\\([A-Z]*\).*/\\1/g\'"
COLOR_FMT = os.popen(str(COLOR_FMT)).read()[:-1]			# cut the "\n" at the end
print (" Color Format =", COLOR_FMT)


if CAM_DRIVER == "ar0144" and COLOR_FMT == "Y":
   v4l2_ctl_param = "auto_exposure=0,auto_exposure_max=720,dynamic_defect_pixel_correction=1"
   CAM_COL_FMT = "Y8_1X8"
   SENSOR_RES = "1280x720"
   OFFSET_SENSOR = "(0,44)"
elif CAM_DRIVER == "ar0144" and COLOR_FMT == "SGRBG":
   v4l2_ctl_param = "auto_exposure=0,auto_exposure_max=720,digital_gain_red=1300,digital_gain_blue=1500,dynamic_defect_pixel_correction=0"
   CAM_COL_FMT = "SGRBG8_1X8"
   SENSOR_RES = "1280x720"
   OFFSET_SENSOR = "(0,44)"
elif CAM_DRIVER == "ar0521" and COLOR_FMT == "Y":
   v4l2_ctl_param = "exposure=2240,analogue_gain=6000,dynamic_defect_pixel_correction=1"
   CAM_COL_FMT = "Y8_1X8"
   SENSOR_RES = "1280x720"
   OFFSET_SENSOR = "(656,612)"
elif CAM_DRIVER == "ar0521" and COLOR_FMT == "SGRBG":
   v4l2_ctl_param = "exposure=2240,analogue_gain=8000,digital_gain_red=1400,digital_gain_blue=1400,dynamic_defect_pixel_correction=1"
   CAM_COL_FMT = "SGRBG8_1X8"
   SENSOR_RES = "1280x720"
   OFFSET_SENSOR = "(656,612)"
else:
   print ("wrong number")
   sys.exit(0)

cam_width = os.popen(str( "echo %s | cut -d\"x\" -f1" % SENSOR_RES)).read()[:-1]
print (" cam_width =", cam_width)
cam_height = os.popen(str( "echo %s | cut -d\"x\" -f2" % SENSOR_RES)).read()[:-1]
print (" cam_width =", cam_height)


# set display
os.environ["DISPLAY"] = ':0.0'


# set video path, camera resolution and ROI position via media-ctl
print ("")
print (" configure CSI1_DEVICE (camera_1 port) with media_ctl")
print (" Note: For use other resolutions,")
print (" see the scripts in path ...\\gstreamer-examples\\...")
print (" ===============================================================")

subprocess.run(["setup-pipeline-csi1", "-f", str(CAM_COL_FMT), "-s", str(SENSOR_RES), "-o", str(OFFSET_SENSOR), "-c", str(SENSOR_RES)])


# set camera parameters via v4l2-ctl
subprocess.run(["v4l2-ctl", "-d", str(CAM1_DEVICE), "-c", str(v4l2_ctl_param)])	# set v4l2 controls


# open videodevice via GStreamer
if CAM_COL_FMT == "SGRBG8_1X8":
   cap = cv2.VideoCapture("v4l2src device="+str(CSI1_DEVICE)+" ! video/x-bayer, format=grbg, width="+str(cam_width)+", height="+str(cam_height)+" ! appsink", cv2.CAP_GSTREAMER)
else:
   cap = cv2.VideoCapture("v4l2src device="+str(CSI1_DEVICE)+" ! video/x-raw, format=GRAY8,depth=8, width="+str(cam_width)+", height="+str(cam_height)+" ! appsink", cv2.CAP_GSTREAMER)


if not cap.isOpened():
    print(" Cannot open camera")
    exit()


# set fullscreen
cv2.namedWindow('phyCAM - Face detection', cv2.WND_PROP_FULLSCREEN)
cv2.setWindowProperty('phyCAM - Face detection',cv2.WND_PROP_FULLSCREEN,cv2.WINDOW_FULLSCREEN)


while(True):
    ret, frame = cap.read()

    if CAM_COL_FMT == "SGRBG8_1X8":
       frame = cv2.cvtColor(frame, cv2.COLOR_BAYER_GB2BGR)

    if fps_average_count == 5:
       new_frame_time = time.time()
       fps = 5/(new_frame_time-prev_frame_time)
       prev_frame_time = new_frame_time
       print ("fps = %3.1f"% fps, end='\r')
       fps_average_count = 0
    fps_average_count += 1
    cv2.putText(frame, "FPS: {:03.1f}".format(fps), TopLeftCornerOfText, font, fontScale, fontColor, lineType)

    if CAM_COL_FMT == "SGRBG8_1X8":
       gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    else:
       gray = frame

    faces = face_cascade.detectMultiScale(gray, scaleFactor=1.3, minNeighbors=3)
    for (x,y,w,h) in faces:
        frame = cv2.rectangle(frame,(x,y),(x+w,y+h),(255,0,0),2)
        roi_gray = gray[y:y+h, x:x+w]
        roi_color = frame[y:y+h, x:x+w]
#        eyes = eye_cascade.detectMultiScale(roi_gray)
#        for (ex,ey,ew,eh) in eyes:
#            cv2.rectangle(roi_color,(ex,ey),(ex+ew,ey+eh),(0,255,0),2)


    # Draw a rectangle around the faces
    for (x, y, w, h) in faces:
        cv2.rectangle(frame, (x, y), (x+w, y+h), (0, 255, 0), 2)

    # Write Number of Faces
    cv2.putText(frame,'/ Number of Faces : ' + str(len(faces)),(180, 30), font, fontScale, fontColor, lineType)

    cv2.imshow('phyCAM - Face detection',frame)
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

cap.release()
cv2.destroyAllWindows()