#!/usr/bin/env python

'''
Sample show VideoCapture via v4l2 interface for PHYTEC phyCAM-M series.
Keys:
    q    - exit
'''

# Python 2/3 compatibility
import cv2
import subprocess
import sys
import os
import time

prev_frame_time = 0
new_frame_time = 0
fps_average_count = 0
fps = 0

font                = cv2.FONT_HERSHEY_SIMPLEX
TopLeftCornerOfText = (10,30)
fontScale           = 1
fontColor           = (255,255,255)
lineType            = 2

print ("")
print (" OpenCV version :  {0}".format(cv2.__version__))

# read /dev/video[x] camera at CSI1/CAM1 via ISP
CSI1_DEVICE = ("/dev/%s" % os.readlink( "/dev/video-csi1" ))
print (" capture device for CSI1 port =", CSI1_DEVICE)

CAM1_DEVICE = ("/dev/%s" % os.readlink( "/dev/cam-csi1" ))
print (" camera device for CSI1 port  =", CAM1_DEVICE)

PHYCAM_L = os.path.exists('/sys/bus/i2c/drivers/ds90ub953/7-0018')
print (" phyCAM-L =", PHYCAM_L)

CAM_DRIVER = ("cat /sys/class/video4linux/%s/name" % os.readlink( "/dev/cam-csi1" ))
CAM_DRIVER = os.popen(str(CAM_DRIVER)).read()[:-1]			# cut the "\n" at the end
CAM_DRIVER = ( "echo %s | cut -d\" \" -f1" % CAM_DRIVER)
CAM_DRIVER = os.popen(str(CAM_DRIVER)).read()[:-1]
print (" Camera Driver Name =", CAM_DRIVER)

COLOR_FMT = "v4l2-ctl -d /dev/cam-csi1 --get-subdev-fmt 0 | grep \"Mediabus Code\" | sed \'s/.*BUS_FMT_\\([A-Z]*\).*/\\1/g\'"
COLOR_FMT = os.popen(str(COLOR_FMT)).read()[:-1]
print (" Color Format =", COLOR_FMT)


cam_width = 1280
cam_height = 720

if CAM_DRIVER == "ar0144" and COLOR_FMT == "Y":
   v4l2_ctl_param = "auto_exposure=1,dynamic_defect_pixel_correction=1"
elif CAM_DRIVER == "ar0144" and COLOR_FMT == "SGRBG":
   v4l2_ctl_param = "auto_exposure=1,digital_gain_red=1000,digital_gain_blue=1000,dynamic_defect_pixel_correction=0"
elif CAM_DRIVER == "ar0521" and COLOR_FMT == "Y":
   v4l2_ctl_param = "exposure=2240,analogue_gain=2400,dynamic_defect_pixel_correction=1"
elif CAM_DRIVER == "ar0521" and COLOR_FMT == "SGRBG":
   v4l2_ctl_param = "exposure=2240,analogue_gain=4000,digital_gain_red=1000,digital_gain_blue=1000,dynamic_defect_pixel_correction=1"
else:
   print ("wrong number")
   sys.exit(0)


# set display
os.environ["DISPLAY"] = ':0.0'


# set XML File 1280x720
subprocess.run([ "sed",  "-i", "s/mode=.*/mode= 0/", "/opt/imx8-isp/bin/Sensor0_Entry.cfg"])


# set video path, camera resolution and ROI position via media-ctl
print ("")
print (" configure CSI1_DEVICE (camera_1 port) with media_ctl")
print (" Note: For use other resolutions,")
print (" see the scripts in path ...\\gstreamer-examples\\...")
print (" ===============================================================")

# set media-ctl for phyCAM-L
if PHYCAM_L == 1:
    subprocess.run(["media-ctl", "-l", "'ds90ub953 7-0018':1->'ds90ub954 2-003d':0[1]"])


# set camera parameters via v4l2-ctl
subprocess.run(["v4l2-ctl", "-d", str(CAM1_DEVICE), "-c", str(v4l2_ctl_param)])	# set v4l2 controls
print (" v4l2_ctl_param =", v4l2_ctl_param)


# open videodevice via GStreamer
cap = cv2.VideoCapture("v4l2src device="+str('/dev/video-csi1')+" ! video/x-raw, format=YUY2, width="+str(cam_width)+", height="+str(cam_height)+" ! appsink", cv2.CAP_GSTREAMER)
#cap = cv2.VideoCapture('/dev/video-csi1', cv2.CAP_V4L)


if not cap.isOpened():
    print(" Cannot open camera")
    exit()

while(True):
    ret, frame = cap.read()

    frame = cv2.cvtColor(frame, cv2.COLOR_YUV2BGRA_YUY2)
    if fps_average_count == 20:
       new_frame_time = time.time()
       fps = 20/(new_frame_time-prev_frame_time)
       prev_frame_time = new_frame_time
       print ("fps = %3.1f"% fps, end='\r')
       fps_average_count = 0
    fps_average_count += 1
    cv2.putText(frame, "FPS: {:03.1f}".format(fps), TopLeftCornerOfText, font, fontScale, fontColor, lineType)

    cv2.imshow('phyCAM - Live Image',frame)
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

cap.release()
cv2.destroyAllWindows()